class PyTreeView:
    def _populate_row(self, iter, pyobj):
        assert(False)  # Derived class must implement

    def set(self, objs):
        self.clear()
        if len(objs):
            for obj in objs:
                iter = self.model.append()
                self._populate_row(iter, obj)
            selection = self.view.get_selection()
            selection.select_path((0,))

    def rows(self):
        return len(self.model)

    def get(self):
        objs = []
        for row in self.model:
            objs.append(row[self._pyobj_index])
        return objs

    def get_row(self, row):
        return self.model[row][self._pyobj_index]

    def clear(self):
        self.model.clear()

    def get_selected(self):
        selection = self.view.get_selection()
        (model, iter) = selection.get_selected()
        if iter:
            row = self.model.get_path(iter)[0]
            obj = model.get_value(iter, self._pyobj_index)
            return (obj, row)
        else:
            return (None, None)

    def delete_selected(self):
        selection = self.view.get_selection()
        (model, iter) = selection.get_selected()
        if iter:
            path = model.get_path(iter)
            i = path[0]

            model.remove(iter)
            # Select next item
            selection.select_path(path)
            # Or if it was last item, select new last item
            if not selection.path_is_selected(path):
               row = path[0]-1
               if row >= 0:
                  selection.select_path((row,))
            return i
        return None

    def move_selected_up(self):
        selection = self.view.get_selection()
        model, selected = selection.get_selected()
        if not selected:
            return ()
        path = model.get_path(selected)
        row = path[0]
        if row == 0:
            return ()
        this = model[row][self._pyobj_index]
        prev = model[row-1][self._pyobj_index]
        self._populate_row(self.model.get_iter( (row-1,) ), this)
        self._populate_row(self.model.get_iter( (row,) ), prev)
        selection.select_path(row-1)
        return (row-1, row)

    def move_selected_down(self):
        selection = self.view.get_selection()
        model, selected = selection.get_selected()
        if not selected:
            return ()
        path = model.get_path(selected)
        row = path[0]
        if row == self.rows()-1:
            return ()
        this = model[row][self._pyobj_index]
        prev = model[row+1][self._pyobj_index]
        self._populate_row(self.model.get_iter( (row+1,) ), this)
        self._populate_row(self.model.get_iter( (row,) ), prev)
        selection.select_path(row+1)
        return (row, row+1)

    def append(self, obj):
        iter = self.model.append()
        self._populate_row(iter, obj)

    def update(self, row, obj):
        self._populate_row(self.model.get_iter( (row,) ), obj)

